/*
          Copyright (C) 1993, 1994, RSNA and Washington University

          The software and supporting documentation for the Radiological
          Society of North America (RSNA) 1993, 1994 Digital Imaging and
          Communications in Medicine (DICOM) Demonstration were developed
          at the
                  Electronic Radiology Laboratory
                  Mallinckrodt Institute of Radiology
                  Washington University School of Medicine
                  510 S. Kingshighway Blvd.
                  St. Louis, MO 63110
          as part of the 1993, 1994 DICOM Central Test Node project for, and
          under contract with, the Radiological Society of North America.

          THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER RSNA NOR
          WASHINGTON UNIVERSITY MAKE ANY WARRANTY ABOUT THE SOFTWARE, ITS
          PERFORMANCE, ITS MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR
          USE, FREEDOM FROM ANY COMPUTER DISEASES OR ITS CONFORMITY TO ANY
          SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF
          THE SOFTWARE IS WITH THE USER.

          Copyright of the software and supporting documentation is
          jointly owned by RSNA and Washington University, and free access
          is hereby granted as a license to use this software, copy this
          software and prepare derivative works based upon this software.
          However, any distribution of this software source code or
          supporting documentation or derivative works (source code and
          supporting documentation) must include the three paragraphs of
          the copyright notice.
*/
/* Copyright marker.  Copyright will be inserted above.  Do not remove */

/*
**				DICOM 93
**		     Electronic Radiology Laboratory
**		   Mallinckrodt Institute of Radiology
**		Washington University School of Medicine
**
** Module Name(s):
**	private functions
** Author, Date:	Stephen M. Moore, 25-Oct-1997
** Intent:		Provide functions for DICOM Directory services
** Last Update:		$Author: smm $, $Date: 1997/01/24 20:33:37 $
** Source File:		$RCSfile: dcm1.c,v $
** Revision:		$Revision: 1.4 $
** Status:		$State: Exp $
*/

static char rcsid[] = "$Revision: 1.4 $ $RCSfile: dcm1.c,v $";

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#ifdef MALLOC_DEBUG
#include "malloc.h"
#endif
#ifdef GCCSUNOS
#include <unistd.h>
#endif

#ifdef _MSC_VER
#else
#include <sys/file.h>
#endif

#ifdef SOLARIS
#include <sys/fcntl.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include "dicom.h"
#include "condition.h"
#include "lst.h"
#include "dicom_uids.h"
#include "dicom_objects.h"
#include "dicom_ddr.h"

static CTNBOOLEAN debug = FALSE;/* Flag for debugging messages to stdout */

static CONDITION addPatientRecord(DCM_OBJECT **patObject, LST_HEAD **lst)
{
  CONDITION cond;
  DDR_PATIENT patientRecord, *patientPtr;
  DCM_ELEMENT required[] = {
    {DCM_PATNAME, DCM_PN, "", 1, sizeof(patientRecord.PatientName), NULL},
    {DCM_PATID, DCM_LO, "", 1, sizeof(patientRecord.PatientID), NULL},
    {DCM_DIRLOWERLEVELOFFSET, DCM_UL, "", 1,
     sizeof(patientRecord.StudyLinkOffset), NULL},
  };
  DCM_FLAGGED_ELEMENT optional[] = {
    {DCM_PATBIRTHDATE, DCM_DA, "", 1, sizeof(patientRecord.BirthDate), NULL, 0, NULL},
    {DCM_PATSEX, DCM_CS, "", 1, sizeof(patientRecord.Sex), NULL, 0, NULL}
  };
  required[0].d.string = patientRecord.PatientName;
  required[1].d.string = patientRecord.PatientID;
  required[2].d.ul = &patientRecord.StudyLinkOffset;
  optional[0].e.d.string = patientRecord.BirthDate;
  optional[0].flagAddress = &patientRecord.optionFlag;
  optional[1].e.d.string = patientRecord.Sex;
  optional[1].flagAddress = &patientRecord.optionFlag;

  memset(&patientRecord, 0, sizeof(patientRecord));
  cond = DCM_ParseObject(patObject, required, 3, optional, 2, NULL);
  if (cond != DCM_NORMAL) {
    return 0;  /* repair */
  }
  patientPtr = malloc(sizeof(*patientPtr));
  if (patientPtr == NULL) {
    return 0;  /* repair */
  }
  memcpy(patientPtr, &patientRecord, sizeof(patientRecord));
  LST_Enqueue(lst, patientPtr);
  return DDR_NORMAL;
}

static CONDITION addStudyRecord(DCM_OBJECT **studyObject, LST_HEAD **lst)
{
  CONDITION cond;
  DDR_STUDY record, *studyPtr;
  DCM_ELEMENT required[] = {
    {DCM_IDSTUDYDATE, DCM_DA, "", 1, sizeof(record.StudyDate), NULL},
    {DCM_IDSTUDYTIME, DCM_TM, "", 1, sizeof(record.StudyTime), NULL},
    {DCM_IDSTUDYDESCRIPTION, DCM_LO, "", 1, sizeof(record.StudyDescription), NULL},
    {DCM_RELSTUDYINSTANCEUID, DCM_UI, "", 1, sizeof(record.StudyInstanceUID), NULL },
    {DCM_RELSTUDYID, DCM_SH, "", 1, sizeof(record.StudyID), NULL },
    {DCM_IDACCESSIONNUMBER, DCM_SH, "", 1, sizeof(record.AccessionNumber), NULL },
    {DCM_DIRLOWERLEVELOFFSET, DCM_UL, "", 1, sizeof(record.SeriesLinkOffset), NULL}
  };

  required[0].d.string = record.StudyDate;
  required[1].d.string = record.StudyTime;
  required[2].d.string = record.StudyDescription;
  required[3].d.string = record.StudyInstanceUID;
  required[4].d.string = record.StudyID;
  required[5].d.string = record.AccessionNumber;
  required[6].d.ul = &record.SeriesLinkOffset;

  memset(&record, 0, sizeof(record));
  cond = DCM_ParseObject(studyObject, required, (int)DIM_OF(required), NULL, 0, NULL);
  if (cond != DCM_NORMAL) {
    return 0;  /* repair */
  }
  studyPtr = malloc(sizeof(*studyPtr));
  if (studyPtr == NULL) {
    return 0;  /* repair */
  }
  memcpy(studyPtr, &record, sizeof(record));
  LST_Enqueue(lst, studyPtr);
  return DDR_NORMAL;
}

static CONDITION addSeriesRecord(DCM_OBJECT **seriesObject, LST_HEAD **lst)
{
  CONDITION cond;
  DDR_SERIES record, *seriesPtr;
  DCM_ELEMENT required[] = {
    {DCM_IDMODALITY, DCM_CS, "", 1, sizeof(record.Modality), NULL },
    {DCM_RELSERIESINSTANCEUID, DCM_UI, "", 1, sizeof(record.SeriesInstanceUID), NULL},
    {DCM_RELSERIESNUMBER, DCM_IS, "", 1, sizeof(record.SeriesNumber), NULL},
    {DCM_DIRLOWERLEVELOFFSET, DCM_UL, "", 1, sizeof(record.LeafLinkOffset), NULL}
  };

  required[0].d.string = record.Modality;
  required[1].d.string = record.SeriesInstanceUID;
  required[2].d.string = record.SeriesNumber;
  required[3].d.ul = &record.LeafLinkOffset;

  memset(&record, 0, sizeof(record));
  cond = DCM_ParseObject(seriesObject, required, (int)DIM_OF(required), NULL, 0, NULL);
  if (cond != DCM_NORMAL) {
    return 0;     /* repair */
  }
  seriesPtr = malloc(sizeof(*seriesPtr));
  if (seriesPtr == NULL) {
    return 0; /* repair */
  }
  memcpy(seriesPtr, &record, sizeof(record));
  LST_Enqueue(lst, seriesPtr);
  return DDR_NORMAL;
}

static void addImageLeafAttributes(DCM_OBJECT **leafObject, DDR_SERIES_LEAF *leaf)
{
  CONDITION cond;
  DCM_ELEMENT required[] = {
    { DCM_RELIMAGENUMBER, DCM_IS, "", 1, sizeof(leaf->specific.image.ImageNumber), NULL}
  };
  required[0].d.string = leaf->specific.image.ImageNumber;
  cond = DCM_ParseObject(leafObject, required, (int)DIM_OF(required), NULL, 0, NULL);
  if (cond != DCM_NORMAL) {
    COND_DumpConditions();
    exit (1); /* repair */
  }
}

static CONDITION addLeafRecord(DCM_OBJECT **leafObject, LST_HEAD **lst)
{
  CONDITION cond;
  DDR_SERIES_LEAF record, *leafPtr;
  DCM_ELEMENT required[] = {
    {DCM_DIRRECORDTYPE, DCM_CS, "", 1, sizeof(record.RecordType), NULL },
    {DCM_DIRREFERENCEDFILEID, DCM_CS, "", 1, sizeof(record.FileID), NULL},
    {DCM_DIRREFSOPCLASSUID, DCM_UI, "", 1, sizeof(record.SOPClassUID), NULL},
    {DCM_DIRREFSOPINSTANCEUID, DCM_UI, "", 1, sizeof(record.SOPInstanceUID), NULL},
    {DCM_DIRREFTRANSFERSYNTAXUID, DCM_UI, "", 1, sizeof(record.TransferSyntaxUID), NULL}
  };

  required[0].d.string = record.RecordType;
  required[1].d.string = record.FileID;
  required[2].d.string = record.SOPClassUID;
  required[3].d.string = record.SOPInstanceUID;
  required[4].d.string = record.TransferSyntaxUID;

  memset(&record, 0, sizeof(record));
  cond = DCM_ParseObject(leafObject, required, (int)DIM_OF(required), NULL, 0, NULL);
  if (cond != DCM_NORMAL) {
    return 0;  /* repair */
  }
  if (strcmp(record.RecordType, "IMAGE") == 0)
    addImageLeafAttributes(leafObject, &record);

  leafPtr = malloc(sizeof(*leafPtr));
  if (leafPtr == NULL) {
    return 0; /* repair */
  }
  memcpy(leafPtr, &record, sizeof(record));
  LST_Enqueue(lst, leafPtr);

  return DDR_NORMAL;
}

U32 offsetNextRecord(DCM_OBJECT **obj)
{
  CONDITION cond;
  U32 offset;
  DCM_ELEMENT e = {DCM_DIRNEXTRECORDOFFSET, DCM_UL, "", 1, sizeof(offset), NULL};

  e.d.ul = &offset;
  cond = DCM_ParseObject(obj, &e, 1, NULL, 0, NULL);
  if (cond != DCM_NORMAL) {
    return 0;  /* repair */
  }
  return offset;
}

CONDITION DDR_GetFileSetID(DCM_OBJECT** obj, DDR_FILESETID* fileSetID)
{
  static long flag;
  DCM_ELEMENT e = {DCM_DIRFILESETID, DCM_CS, "", 1, DICOM_CS_LENGTH+1, NULL};
  DCM_FLAGGED_ELEMENT optionalElements[] = {
    {DCM_DIRFILESETDESCRFILEID, DCM_CS, "", 1, DICOM_CS_LENGTH+1, NULL, 0, &flag},
    {DCM_DIRSPECIFICCHARACTER, DCM_CS, "", 1, DICOM_CS_LENGTH+1, NULL, 0, &flag}
  };

  CONDITION cond;

  memset(fileSetID, 0, sizeof(fileSetID));
  e.d.string = fileSetID->FileSetID;
  optionalElements[0].e.d.string = fileSetID->DescriptorFileID;
  optionalElements[1].e.d.string = fileSetID->SpecificCharacterSet;

  cond = DCM_ParseObject(obj, &e, 1,
			 optionalElements, (int)DIM_OF(optionalElements), NULL);
  if (cond != DCM_NORMAL)
    return DDR_ERROR;

  return DDR_NORMAL;
}


CONDITION DDR_RootDirectory(DCM_OBJECT** obj)
{
  U32 offset = 0;
  DCM_ELEMENT e = {DCM_DIRFIRSTOFFSET, DCM_UL, "", 1, sizeof(offset), NULL};

  CONDITION cond;

#if 0
  e.d.ul = &offset;

  cond = DCM_ParseObject(obj, &e, 1, NULL, 0, NULL);
  if (cond != DCM_NORMAL)
    return DDR_ERROR;

  cond = DCM_GetSequenceByOffset(obj, DCM_DIRRECORDSEQUENCE, offset);
#endif

  return DDR_NORMAL;
}

CONDITION DDR_GetPatientList(DCM_OBJECT**obj, LST_HEAD** lst)
{
  U32 offset = 0;
  DCM_ELEMENT e = {DCM_DIRFIRSTOFFSET, DCM_UL, "", 1, sizeof(offset), NULL};
  DCM_OBJECT *patObject;
  CONDITION cond;

  e.d.ul = &offset;

  cond = DCM_ParseObject(obj, &e, 1, NULL, 0, NULL);
  if (cond != DCM_NORMAL)
    return DDR_ERROR;


  while (offset != 0) {
    cond = DCM_GetSequenceByOffset(obj, DCM_DIRRECORDSEQUENCE, offset,
				   &patObject);
    if (cond != DCM_NORMAL) {
      exit(1); /* repair */
    }
    addPatientRecord(&patObject, lst);
    offset = offsetNextRecord(&patObject);
  }
  return DDR_NORMAL;
}

CONDITION DDR_GetStudyList(DCM_OBJECT **object, const char* patientID,
			   LST_HEAD** lst)
{
  LST_HEAD *patientList;
  DDR_PATIENT *pPtr;
  CONDITION cond;
  U32 offset;
  DCM_OBJECT *studyObject;

  patientList = LST_Create();
  cond = DDR_GetPatientList(object, &patientList);
  pPtr = LST_Dequeue(&patientList);

  while(pPtr != NULL) {
    if (strcmp(pPtr->PatientID, patientID) == 0) {
      offset = pPtr->StudyLinkOffset;
      while(offset != 0) {
	cond = DCM_GetSequenceByOffset(object, DCM_DIRRECORDSEQUENCE,
				       offset, &studyObject);
	if (cond != DCM_NORMAL) {
	  exit(1); /* repair */
	}
	addStudyRecord(&studyObject, lst);
	offset = offsetNextRecord(&studyObject);
      }
    }
    free(pPtr);
    pPtr = LST_Dequeue(&patientList);
  }
  LST_Destroy(&patientList);
  return DDR_NORMAL;
}

CONDITION DDR_GetSeriesList(DCM_OBJECT **object, const char* patientID,
			    const char* studyInstanceUID, LST_HEAD **seriesList)
{
  LST_HEAD *studyList;
  DDR_STUDY *sPtr;
  CONDITION cond;
  U32 offset;
  DCM_OBJECT *seriesObject;

  studyList = LST_Create();
  cond = DDR_GetStudyList(object, patientID, &studyList);
  /* repair */

  sPtr = LST_Dequeue(&studyList);
  while (sPtr != NULL) {
    if (strcmp(sPtr->StudyInstanceUID, studyInstanceUID) == 0) {
      offset = sPtr->SeriesLinkOffset;
      while (offset != 0) {
	cond = DCM_GetSequenceByOffset(object, DCM_DIRRECORDSEQUENCE,
				       offset, &seriesObject);
	if (cond != DCM_NORMAL) {
	  exit(1); /* repair */
	}
	addSeriesRecord(&seriesObject, seriesList);
	offset = offsetNextRecord(&seriesObject);
      }
    }
    free(sPtr);
    sPtr = LST_Dequeue(&studyList);
  }
  LST_Destroy(&studyList);
  return DDR_NORMAL;
}

CONDITION DDR_GetSeriesLeafList(DCM_OBJECT **object, const char* patientID,
				const char* studyInstanceUID,
				const char* seriesInstanceUID,
				LST_HEAD **leafList)
{
  LST_HEAD *seriesList;
  DDR_SERIES *sPtr;
  CONDITION cond;
  U32 offset;
  DCM_OBJECT *leafObject;

  seriesList = LST_Create();
  cond = DDR_GetSeriesList(object, patientID, studyInstanceUID, &seriesList);
  /* repair */

  sPtr = LST_Dequeue(&seriesList);
  while (sPtr != NULL) {
    if (strcmp(sPtr->SeriesInstanceUID, seriesInstanceUID) == 0) {
      offset = sPtr->LeafLinkOffset;
      while (offset != 0) {
	cond = DCM_GetSequenceByOffset(object, DCM_DIRRECORDSEQUENCE,
				       offset, &leafObject);
	if (cond != DCM_NORMAL) {
	  exit(1); /* repair */
	}
	addLeafRecord(&leafObject, leafList);
	offset = offsetNextRecord(&leafObject);
      }
    }
    free(sPtr);
    sPtr = LST_Dequeue(&seriesList);
  }
  LST_Destroy(&seriesList);
  return DDR_NORMAL;
}
